package cd.casic.ci.process.api.process.utils;

import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.common.pipeline.container.*;
import cd.casic.ci.common.pipeline.pojo.BuildFormProperty;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.process.api.process.pojo.setting.PipelineSettingVersion;
import cn.hutool.core.collection.CollUtil;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author by mianbin
 * @Classname PipelineVersionUtils
 * @Description TODO
 * @Date 2025/4/1 19:52
 */
public class PipelineVersionUtils {

    public String getVersionNameByModel(
            int currPipelineVersion,
            int currTriggerVersion,
            int settingVersion,
            int versionNum,
            Model originModel,
            Model newModel
    ) {
        int pipelineVersion = getPipelineVersion(currPipelineVersion, originModel, newModel);
        pipelineVersion = Math.max(pipelineVersion, 1);
        int triggerVersion = getTriggerVersion(currTriggerVersion, originModel, newModel);
        triggerVersion = Math.max(triggerVersion, 1);
        return "V" + versionNum + "(P" + pipelineVersion + ".T" + triggerVersion + "." + settingVersion + ")";
    }

    public static String getVersionName(Integer versionNum, Integer pipelineVersion, Integer triggerVersion, Integer settingVersion) {
        boolean isNull = versionNum == null || pipelineVersion == null || triggerVersion == null || settingVersion == null;
        return isNull ? null : "V" + versionNum + "(P" + pipelineVersion + ".T" + triggerVersion + "." + settingVersion + ")";
    }

    public int getTriggerVersion(int currVersion, Model originModel, Model newModel) {
        try {
            boolean changed = false;
            TriggerContainer originTrigger = (TriggerContainer) originModel.getStages().get(0).getContainers().get(0);
            originTrigger.setParams(Collections.emptyList());
            TriggerContainer newTrigger = (TriggerContainer) newModel.getStages().get(0).getContainers().get(0);
            newTrigger.setParams(Collections.emptyList());
            if (originTrigger.equals(newTrigger)) {
                List<Element> originElements = originTrigger.getElements();
                List<Element> newElements = newTrigger.getElements();
                for (int i = 0; i < originElements.size(); i++) {
                    Element origin = originElements.get(i);
                    Element newElement = newElements.get(i);
                    if (!origin.equals(newElement)) {
                        changed = true;
                    }
                    if (origin.elementEnabled() != newElement.elementEnabled()) {
                        changed = true;
                    }
                }
            } else {
                changed = true;
            }
            return changed ? currVersion + 1 : currVersion;
        } catch (Throwable ignore) {
            return currVersion + 1;
        }
    }

    public int getPipelineVersion(
            int currVersion,
            Model originModel,
            Model newModel
    ) {
        List<Stage> originStages = originModel.getStages().subList(1, originModel.getStages().size());
        List<Stage> newStages = newModel.getStages().subList(1, newModel.getStages().size());
        List<BuildFormProperty> originParams = ((TriggerContainer) originModel.getStages().get(0).getContainers().get(0)).getParams();
        List<BuildFormProperty> newParams = ((TriggerContainer) newModel.getStages().get(0).getContainers().get(0)).getParams();
        return CollUtil.disjunction(originStages, newStages).size() == 0 && originParams.equals(newParams) ? currVersion : currVersion + 1;
    }

    public int getSettingVersion(
            int currVersion,
            PipelineSettingVersion originSetting,
            PipelineSettingVersion newSetting
    ) {
        return originSetting.equals(newSetting) ? currVersion : currVersion + 1;
    }

    private boolean differ(List<Stage> thisStages, List<Stage> otherStages) {
        if (!thisStages.equals(otherStages) && thisStages.size() != otherStages.size()) {
            return false;
        }
        for (int sIndex = 0; sIndex < thisStages.size(); sIndex++) {
            Stage thisStage = thisStages.get(sIndex);
            Stage otherStage = otherStages.get(sIndex);
            if (!thisStage.equals(otherStage) || thisStage.getContainers().size() != otherStage.getContainers().size()
                    || !thisStage.getCheckIn().equals(otherStage.getCheckIn())
                    || !thisStage.getCheckOut().equals(otherStage.getCheckOut())
                    || !thisStage.getStageControlOption().equals(otherStage.getStageControlOption())) {
                return false;
            }
            List<Container> thisContainers = thisStage.getContainers();
            List<Container> otherContainers = otherStage.getContainers();
            for (int cIndex = 0; cIndex < thisContainers.size(); cIndex++) {
                Container thisContainer = thisContainers.get(cIndex);
                Container otherContainer = otherContainers.get(cIndex);
                if (!thisContainer.equals(otherContainer) && thisContainer.getElements().size() != otherContainer.getElements().size()) {
                    return false;
                }
                if (thisContainer instanceof VMBuildContainer && otherContainer instanceof VMBuildContainer) {
                    VMBuildContainer thisVMBuild = (VMBuildContainer) thisContainer;
                    VMBuildContainer otherVMBuild = (VMBuildContainer) otherContainer;
                    if (!thisVMBuild.equals(otherVMBuild) || !thisVMBuild.getDispatchType().equals(otherVMBuild.getDispatchType())
                            || !thisVMBuild.getJobControlOption().equals(otherVMBuild.getJobControlOption())) {
                        return false;
                    }
                } else if (thisContainer instanceof NormalContainer && otherContainer instanceof NormalContainer) {
                    NormalContainer thisNormal = (NormalContainer) thisContainer;
                    NormalContainer otherNormal = (NormalContainer) otherContainer;
                    if (!thisNormal.equals(otherNormal) || !thisNormal.getJobControlOption().equals(otherNormal.getJobControlOption())) {
                        return false;
                    }
                } else {
                    return false;
                }
                List<Element> thisElements = thisContainer.getElements();
                List<Element> otherElements = otherContainer.getElements();
                for (int eIndex = 0; eIndex < thisElements.size(); eIndex++) {
                    Element thisElement = thisElements.get(eIndex);
                    Element otherElement = otherElements.get(eIndex);
                    if (!thisElement.equals(otherElement)) {
                        return false;
                    }
                    if (!thisElement.getAdditionalOptions().equals(otherElement.getAdditionalOptions())) {
                        return false;
                    }

                    if (CollUtil.disjunction(thisElements, otherElements).size() == 0) {
                        return false;
                    }
                }
            }
        }
        return true;
    }

    public boolean differ(Element thisElement, Element otherElement) {
        if (thisElement.getClass() != otherElement.getClass()) {
            return true;
        }
        java.lang.reflect.Field[] v1Fields = thisElement.getClass().getDeclaredFields();
        java.lang.reflect.Field[] v2Fields = otherElement.getClass().getDeclaredFields();
        if (v1Fields.length != v2Fields.length) {
            return true;
        }
        java.util.Map<String, Object> v1Map = new java.util.HashMap<>();
        java.util.Map<String, Object> v2Map = new java.util.HashMap<>();
        try {
            for (java.lang.reflect.Field field : v1Fields) {
                field.setAccessible(true);
                v1Map.put(field.getName(), field.get(thisElement));
            }
            for (java.lang.reflect.Field field : v2Fields) {
                field.setAccessible(true);
                v2Map.put(field.getName(), field.get(otherElement));
            }
            if (v1Map.size() != v2Map.size()) {
                return true;
            }
            for (Map.Entry<String, Object> entry : v1Map.entrySet()) {
                String key = entry.getKey();
                Object value = entry.getValue();
                if (!v2Map.containsKey(key)) {
                    return true;
                }
                if (!v2Map.get(key).equals(value)) {
                    return true;
                }
            }
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
        return false;
    }
}
